#include "stdafx.h"
#include "EasyPadWrapper.h"
#include "EasyPad.h"
#include "PadButtonRectangular.h"
#include "PadCalPoint.h"
#include "PadRegionRectangular.h"

EasyPad::EasyPad(EasyPadWrapper& _parent)
: parent(_parent)
, padBoundary(*new PadRegionRectangular)
{
	wasInPadZone = true;
}
EasyPad::EasyPad()
: parent(*new EasyPadWrapper)
, padBoundary(*new PadRegionRectangular)
{
	wasInPadZone = true;
}

EasyPad::~EasyPad(void)
{
	delete &padBoundary;
}

EasyPad::EasyPad(EasyPadWrapper& _parent, const EasyPad& source)
: parent(_parent)
, padBoundary(*new PadRegionRectangular)
{
	operator=(source);
}
void EasyPad::operator=(const EasyPad& source)
{
	padBoundary = source.padBoundary;
	instToPad = source.instToPad;
	padRect = source.padRect;
	Buttons.DeleteAll();
	Buttons.AppendCopy(source.Buttons);
	CalPoints.DeleteAll();
	CalPoints.AppendCopy(source.CalPoints);
}

// build pad...
void EasyPad::SetBoundary(const Vector& topLeft, const Vector& bottomRight)
{
	padBoundary.topLeft = topLeft;
	padBoundary.bottomRight = bottomRight;
}
void EasyPad::SetHeights(const double& workingHeight, const double& hotHeight)
{
	padBoundary.SetHeights(workingHeight, hotHeight);
}
DWORD_PTR EasyPad::AddRectangularButton(const Vector& topLeft, const Vector& bottomRight, const double& pressHeight)
{
	PadButtonRectangular& button = *new PadButtonRectangular;
	button.SetLocationInPad(topLeft, bottomRight);
	button.SetPressHeight(pressHeight);
	return Buttons.Add(button);
}

// inputs...
bool EasyPad::Calibrate()
{
	if(CalPoints.GetSize() < 3)
		return false;

	Vector xAx,yAx,zAx;

	xAx = CalPoints[2] - CalPoints[1];
	yAx = CalPoints[0] - CalPoints[1];
	zAx.Cross(xAx, yAx);
	yAx.Cross(zAx, xAx);

	xAx.Normalize();
	yAx.Normalize();

	Transform padToInst;
	padToInst.SetTranslation(CalPoints[1]);
	padToInst.SetAxes(xAx, yAx, zAx);

	// now move the pad down by the inst's planar offset...
	Vector shift(zAx);
	shift.Normalize();
	shift *= -1.0*CalPoints[0].instPlanarOffset;
	Vector newOrig(padToInst.GetTranslation());
	newOrig += shift;
	padToInst.SetTranslation(newOrig);


	padToInst.Invert(&instToPad);

	return true;
}

bool EasyPad::IsCalibrated()
{
	return CalPoints.GetSize() >= 3 ? true : false;
}

void EasyPad::CheckPosition(Vector instXYZ, double instOffset)
{
	Vector ptInPad;
	instToPad.Multiply(instXYZ, &ptInPad);
	ptInPad.z -= instOffset;

	if(padBoundary.PointInRegion(ptInPad))
	{
		wasInPadZone = true;

		for(DWORD_PTR i=0; i<Buttons.GetSize(); i++)
		{
			bool enterButtonZone = false;
			bool exitButtonZone = false;
			bool buttonDown = false;
			bool pressed = Buttons[i].CheckForPress(ptInPad, enterButtonZone, exitButtonZone, buttonDown);
			if(pressed)
			{
				parent.OnPadButtonHit(i);
				SetAllButtonsUnPressed();
				return;
			}
			if(buttonDown)
			{
				SetAllButThisButtonUnPressed(i);
			}
			else if(enterButtonZone)
			{
				parent.OnEnterButtonZone(i);
				return;
			}
			else if(exitButtonZone)
			{
				parent.OnExitButtonZone(i);
				return;
			}
		}

		if(ptInPad.z > padBoundary.workingHeight)
		{
			if(padBoundary.inWorkingZone || padBoundary.inHotZone)
			{
				parent.OnExitPadZone();
			}
			padBoundary.inWorkingZone = false;
			padBoundary.inHotZone = false;
			SetAllButtonZonesDead();
		}
		else if(ptInPad.z < padBoundary.hotHeight && padBoundary.inHotZone)
		{
			if(!padBoundary.inWorkingZone)
			{
				padBoundary.inWorkingZone = true;
				parent.OnEnterPadWorkingZone();
			}
		}
		else if(ptInPad.z < padBoundary.hotHeight && !padBoundary.inHotZone)
		{
			padBoundary.inHotZone = true;
			parent.OnEnterPadHotZone();
		}
		else if(ptInPad.z < padBoundary.workingHeight && !padBoundary.inWorkingZone)
		{
			padBoundary.inWorkingZone = true;
			parent.OnEnterPadWorkingZone();
		}
		else if(ptInPad.z > padBoundary.hotHeight)
		{
			padBoundary.inHotZone = false;
		}
	}
	else if(wasInPadZone)
	{
		wasInPadZone = false;
		if(padBoundary.inWorkingZone || padBoundary.inHotZone)
		{
			parent.OnExitPadZone();
		}
		padBoundary.inWorkingZone = false;
		padBoundary.inHotZone = false;
		SetAllButtonZonesDead();
	}
}

// utility
void EasyPad::SetAllButtonZonesDead()
{
	for(DWORD_PTR i=0; i<Buttons.GetSize(); i++)
	{
		Buttons[i].SetZonesDead();
	}
	parent.SetAllButtonZonesDead();	// be sure to un-hilight them if there's UI showing
}
void EasyPad::SetAllButtonsUnPressed()
{
	for(DWORD_PTR i=0; i<Buttons.GetSize(); i++)
	{
		Buttons[i].SetUnPressed();
	}
}
void EasyPad::SetAllButThisButtonUnPressed(const DWORD_PTR& idx)
{
	for(DWORD_PTR i=0; i<Buttons.GetSize(); i++)
	{
		if(i != idx)
		{
			Buttons[i].SetUnPressed();
			TRACE1("button %d set unpressed\n", i);
		}
	}
}

void EasyPad::Serialize(CArchive& ar) 
{
	BYTE Schema;
	BYTE LoadSchema = 0;
	Schema = 1;
	if (ar.IsStoring())
	{
		ar << Schema;
		instToPad.Serialize(ar);
		CalPoints.SerializeObArray(ar);
	}
	else
	{
		ar >> LoadSchema;
		instToPad.Serialize(ar);
		CalPoints.SerializeObArray(ar);
	}
}
IMPLEMENT_SERIAL( EasyPad, CObject, 0 );