#include "stdafx.h"
#include "persistentwnd.h"

// these routines allow you to add pos/size persistence to window(s) in your application.
//
// you can choose to save/restore just size, just pos, or both depending on the functions
// you use.
//
// the routines take the same arguments, and so they are documented here:
//
// section			which registry subkey you want the persistence information stored.
//					you have to have already called CWinApp::SetRegistryKey() before
//					using these routines!  This is normally called in your InitInstance().
//					If you try and use these routines before doing this, they will not
//					properly store the information.
//
// window			a unique name to identify the window.  window titlebars can change,
//					so this allows you to specify a unique name for the window.  this information
//					is needed so that persistence info can be stored for multiple windows.
//
// pWnd				the window in question.
//
// checkVisible		before setting the window position, make sure that its completely visible
//					on the screen.  if not, don't set the position.  this parameter defaults
//					to TRUE if you don't pass it.


// standard sanity asserts!
#define STD_ASSERTS \
	ASSERT( section ); \
	ASSERT( window ); \
	ASSERT( pWnd ); \
	ASSERT( (::IsWindow(pWnd->GetSafeHwnd())) ); \
	ASSERT( (lstrlen(section) > 0) ); \
	ASSERT( (lstrlen(window) > 0) );


// crazy pos/size values you will never use!  if these get returned
// we assume no entry was stored in the registry
#define CRAZY_POS -25000
#define CRAZY_SIZE -25000


void PersistentWnd::SaveWndRect(LPCTSTR section, LPCTSTR window, CWnd * pWnd)
{
	STD_ASSERTS

	SaveWndPos(section,window,pWnd);
	SaveWndSize(section,window,pWnd);
}


void PersistentWnd::SaveWndPos(LPCTSTR section, LPCTSTR window, CWnd * pWnd)
{
	STD_ASSERTS

	CRect wr;
	pWnd->GetWindowRect(wr);

	_SaveWndPos(section,window,wr.TopLeft());
}


void PersistentWnd::SaveWndSize(LPCTSTR section, LPCTSTR window, CWnd * pWnd)
{
	STD_ASSERTS

	CRect wr;
	pWnd->GetWindowRect(wr);
	
	_SaveWndSize(section,window,wr.Size());
}


void PersistentWnd::RestoreWndRect(LPCTSTR section, LPCTSTR window, CWnd * pWnd, BOOL checkVisible)
{
	STD_ASSERTS

	if (checkVisible)
	{
		CPoint pos;
		CSize size;
		if (_LoadWndPos(section,window,pos))
		{
			if (_LoadWndSize(section,window,size))
			{
				CRect pr;
				pr.left = pos.x;
				pr.top = pos.y;
				pr.right = pr.left + size.cx;
				pr.bottom = pr.top + size.cy;

				if (!WndRectValid(pr))
					return;
			}
		}
	}

	RestoreWndPos(section,window,pWnd,FALSE);
	RestoreWndSize(section,window,pWnd,FALSE);
}


void PersistentWnd::RestoreWndPos(LPCTSTR section, LPCTSTR window, CWnd * pWnd, BOOL checkVisible)
{
	STD_ASSERTS

	CPoint pos;
	if (_LoadWndPos(section,window,pos))
	{
		if (checkVisible)
		{
			CRect wr;
			pWnd->GetWindowRect(wr);
			CRect pr;
			pr.left = pos.x;
			pr.top = pos.y;
			pr.right = pr.left + wr.Width();
			pr.bottom = pr.top + wr.Height();

			if (!WndRectValid(pr))
				return;
		}

		pWnd->SetWindowPos(0,pos.x,pos.y,0,0,SWP_NOZORDER|SWP_NOSIZE);
	}
}


void PersistentWnd::RestoreWndSize(LPCTSTR section, LPCTSTR window, CWnd * pWnd, BOOL checkVisible)
{
	STD_ASSERTS

	CSize size;
	if (_LoadWndSize(section,window,size))
	{
		if (checkVisible)
		{
			CRect wr;
			pWnd->GetWindowRect(wr);
			CRect pr;
			pr.left = wr.left;
			pr.top = wr.top;
			pr.right = pr.left + size.cx;
			pr.bottom = pr.top + size.cy;

			if (!WndRectValid(pr))
				return;
		}

		pWnd->SetWindowPos(0,0,0,size.cx,size.cy,SWP_NOZORDER|SWP_NOMOVE);
	}
}


//-------------------------------------- routines to load/save window information


BOOL PersistentWnd::_LoadWndPos(LPCTSTR section, LPCTSTR window, CPoint & pos)
{
	CWinApp * pApp = AfxGetApp();

	CString w;
	w.Format("%s_X", window);
	int x = (int)pApp->GetProfileInt(section,w,CRAZY_POS);
	w.Format("%s_Y", window);
	int y = (int)pApp->GetProfileInt(section,w,CRAZY_POS);

	if ( (x == CRAZY_POS) || (y == CRAZY_POS) )
		return FALSE;

	pos.x = x;
	pos.y = y;
	return TRUE;
}


BOOL PersistentWnd::_LoadWndSize(LPCTSTR section, LPCTSTR window, CSize & size)
{
	CWinApp * pApp = AfxGetApp();

	CString w;
	w.Format("%s_CX", window);
	int cx = (int)pApp->GetProfileInt(section,w,CRAZY_SIZE);
	w.Format("%s_CY", window);
	int cy = (int)pApp->GetProfileInt(section,w,CRAZY_SIZE);

	if ( (cx == CRAZY_SIZE) || (cy == CRAZY_SIZE) )
		return FALSE;

	size.cx = cx;
	size.cy = cy;
	return TRUE;
}


void PersistentWnd::_SaveWndPos(LPCTSTR section, LPCTSTR window, CPoint pos)
{
	CWinApp * pApp = AfxGetApp();

	CString w;
	w.Format("%s_X", window);
	pApp->WriteProfileInt(section,w,pos.x);
	w.Format("%s_Y", window);
	pApp->WriteProfileInt(section,w,pos.y);
}


void PersistentWnd::_SaveWndSize(LPCTSTR section, LPCTSTR window, CSize size)
{
	CWinApp * pApp = AfxGetApp();

	CString w;
	w.Format("%s_CX", window);
	pApp->WriteProfileInt(section,w,size.cx);
	w.Format("%s_CY", window);
	pApp->WriteProfileInt(section,w,size.cy);
}


//-------------------------------------- determine if the proposed window rect is valid


BOOL PersistentWnd::WndRectValid(CRect wr)
{
	CRect sr;
	sr.left = 0;
	sr.top = 0;
	sr.right = GetSystemMetrics(SM_CXSCREEN);
	sr.bottom = GetSystemMetrics(SM_CYSCREEN);
	
	wr.NormalizeRect();
	sr.NormalizeRect();

	CRect ir;
	ir.IntersectRect(wr,sr);

	return
		( (ir.Width() == wr.Width()) &&
			(ir.Height() == wr.Height()) );
}
